@@ -1,6 +1,6 @@
-# pushover 0.2
+# pushover 0.4
 #
-# Copyright (C) 2013-2014  Thibaut Horel <thibaut.horel@gmail.com>
+# Copyright (C) 2013-2018  Thibaut Horel <thibaut.horel@gmail.com>
 
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -29,6 +29,7 @@ MESSAGE_URL = BASE_URL + "messages.json"
 USER_URL = BASE_URL + "users/validate.json"
 SOUND_URL = BASE_URL + "sounds.json"
 RECEIPT_URL = BASE_URL + "receipts/"
+GLANCE_URL = BASE_URL + "glances.json"
 
 SOUNDS = None
 TOKEN = None
@@ -98,12 +99,12 @@ class Request:
     a :class:`RequestError` exception when the request is rejected.
     """
 
-    def __init__(self, request_type, url, payload):
+    def __init__(self, request_type, url, payload, files={}):
         if not TOKEN:
             raise InitError
 
         payload["token"] = TOKEN
-        request = getattr(requests, request_type)(url, params=payload)
+        request = getattr(requests, request_type)(url, params=payload, files=files)
         self.answer = request.json()
         if 400 <= request.status_code < 500:
             raise RequestError(self.answer["errors"])
@@ -124,15 +125,17 @@ class MessageRequest(Request):
     :func:`poll` function.
     """
 
-    def __init__(self, payload):
-        Request.__init__(self, "post", MESSAGE_URL, payload)
+    def __init__(self, payload, files):
+        Request.__init__(self, "post", MESSAGE_URL, payload, files)
         self.receipt = None
         if payload.get("priority", 0) == 2:
             self.receipt = self.answer["receipt"]
-        self.parameters = ["expired", "called_back", "acknowledged"]
-        for parameter in self.parameters:
-            setattr(self, parameter, False)
-            setattr(self, parameter + "_at", 0)
+        self.parameters = {"expired": "expires_at",
+                           "called_back": "called_back_at",
+                           "acknowledged": "acknowledged_at"}
+        for param, when in self.parameters.iteritems():
+            setattr(self, param, False)
+            setattr(self, when, 0)
 
     def poll(self):
         """If the message request has a priority of 2, Pushover will keep
@@ -147,20 +150,47 @@ class MessageRequest(Request):
         acknowledged, so that a typical handling of a priority-2 notification
         can look like this::
 
-            request = client.send_message("Urgent notification", priority=2)
-            while not request.poll():
+            request = client.send_message("Urgent notification", priority=2,
+                                          expire=120, retry=60)
+            while request.poll():
                 # do something
                 time.sleep(5)
+
+            print request.acknowledged_at, request.acknowledged_by
         """
         if (self.receipt and not any(getattr(self, parameter)
                                      for parameter in self.parameters)):
             request = Request("get", RECEIPT_URL + self.receipt + ".json", {})
-            for parameter in self.parameters:
-                setattr(self, parameter, request.answer[parameter])
-                setattr(self, parameter + "_at",
-                        request.answer[parameter + "_at"])
+            for param, when in self.parameters.iteritems():
+                setattr(self, param, bool(request.answer[param]))
+                setattr(self, when, request.answer[when])
+            for param in ["last_delivered_at", "acknowledged_by",
+                          "acknowledged_by_device"]:
+                setattr(self, param, request.answer[param])
             return request
 
+    def cancel(self):
+        """If the message request has a priority of 2, Pushover will keep
+        sending the same notification until it either reaches its ``expire``
+        value or is aknowledged by the client. Calling the :func:`cancel`
+        function will cancel the notification early.
+        """
+        if (self.receipt and not any(getattr(self, parameter)
+                                     for parameter in self.parameters)):
+            request = Request("post", RECEIPT_URL + self.receipt
+                              + "/cancel.json", {})
+            return request
+
+
+class GlanceRequest(Request):
+    """Class representing a glance request to the Pushover API. This is
+    a heavily simplified version of the MessageRequest class, with all
+    polling-related features removed.
+    """
+
+    def __init__(self, payload):
+        Request.__init__(self, "post", GLANCE_URL, payload)
+
 
 class Client:
     """This is the main class of the module. It represents a specific Pushover
@@ -179,7 +209,7 @@ class Client:
 
     def __init__(self, user_key=None, device=None, api_token=None,
                  config_path="~/.pushoverrc", profile="Default"):
-        params = _get_config(profile, config_path, user_key, api_token)
+        params = _get_config(profile, config_path, user_key, api_token, device)
         self.user_key = params["user_key"]
         if not self.user_key:
             raise UserError
@@ -204,21 +234,25 @@ class Client:
         self.devices = request.answer["devices"]
         return True
 
-    def send_message(self, message, **kwords):
+    def send_message(self, message, attachment=None, **kwords):
         """Send a message to the user. It is possible to specify additional
         properties of the message by passing keyword arguments. The list of
         valid keywords is ``title, priority, sound, callback, timestamp, url,
-        url_title, device, retry and expire`` which are described in the
+        url_title, device, retry, expire and html`` which are described in the
         Pushover API documentation. For convenience, you can simply set
         ``timestamp=True`` to set the timestamp to the current timestamp.
 
+        An image can be attached to a message by passing a file-like object
+        with the `attachment` keyword argument.
+
         This method returns a :class:`MessageRequest` object.
         """
         valid_keywords = ["title", "priority", "sound", "callback",
                           "timestamp", "url", "url_title", "device",
-                          "retry", "expire"]
+                          "retry", "expire", "html"]
 
         payload = {"message": message, "user": self.user_key}
+        files = {'attachment': attachment} if attachment else {}
         if self.device:
             payload["device"] = self.device
 
@@ -238,11 +272,36 @@ class Client:
             elif value:
                 payload[key] = value
 
-        return MessageRequest(payload)
+        return MessageRequest(payload, files)
+
+    def send_glance(self, text=None, **kwords):
+        """Send a glance to the user. The default property is ``text``,
+        as this is used on most glances, however a valid glance does not
+        need to require text and can be constructed using any combination
+        of valid keyword properties. The list of valid keywords is ``title,
+        text, subtext, count and percent`` which are  described in the
+        Pushover Glance API documentation.
+
+        This method returns a :class:`GlanceRequest` object.
+        """
+        valid_keywords = ["title", "text", "subtext", "count", "percent"]
+
+        payload = {"user": self.user_key}
+        if text:
+            payload["text"] = text
+        if self.device:
+            payload["device"] = self.device
+
+        for key, value in kwords.iteritems():
+            if key not in valid_keywords:
+                raise ValueError("{0}: invalid message parameter".format(key))
+            payload[key] = value
+
+        return GlanceRequest(payload)
 
 
 def _get_config(profile='Default', config_path='~/.pushoverrc',
-                user_key=None, api_token=None):
+                user_key=None, api_token=None, device=None):
     config_path = os.path.expanduser(config_path)
     config = RawConfigParser()
     config.read(config_path)
@@ -255,6 +314,8 @@ def _get_config(profile='Default', config_path='~/.pushoverrc',
         params["user_key"] = user_key
     if api_token:
         params["api_token"] = api_token
+    if device:
+        params["device"] = device
 
     if not TOKEN:
         init(params["api_token"])
@@ -273,7 +334,9 @@ For more details and bug reports, see: https://github.com/Thibauth/python-pushov
     parser.add_argument("--user-key", "-u", help="Pushover user key")
     parser.add_argument("message", help="message to send")
     parser.add_argument("--title", "-t", help="message title")
-    parser.add_argument("--priority", "-p", help="message priority")
+    parser.add_argument("--priority", "-p", help="message priority (-1, 0, 1 or 2)", type=int)
+    parser.add_argument("--retry", "-r", help="how often (in seconds) the Pushover servers will send the same notification to the user", type=int)
+    parser.add_argument("--expire", "-e", help="how many seconds your notification will continue to be retried for (every retry seconds).", type=int)
     parser.add_argument("--url", help="additional url")
     parser.add_argument("--url-title", help="additional url title")
     parser.add_argument("-c", "--config", help="configuration file\
@@ -284,17 +347,21 @@ For more details and bug reports, see: https://github.com/Thibauth/python-pushov
     parser.add_argument("--version", "-v", action="version",
                         help="output version information and exit",
                         version="""
-%(prog)s 0.2
-Copyright (C) 2013-2014 Thibaut Horel <thibaut.horel@gmail.com>
+%(prog)s 0.4
+Copyright (C) 2013-2018 Thibaut Horel <thibaut.horel@gmail.com>
 License GPLv3+: GNU GPL version 3 or later <http://gnu.org/licenses/gpl.html>.
 This is free software: you are free to change and redistribute it.
 There is NO WARRANTY, to the extent permitted by law.""")
 
     args = parser.parse_args()
+    if args.priority and args.priority==2 and (args.retry is None or args.expire is None):
+        parser.error("priority of 2 requires expire and retry")
+
     Client(args.user_key, None, args.api_token, args.config,
            args.profile).send_message(args.message, title=args.title,
                                       priority=args.priority, url=args.url,
-                                      url_title=args.url_title, timestamp=True)
+                                      url_title=args.url_title, timestamp=True,
+                                      retry=args.retry,expire=args.expire)
 
 if __name__ == "__main__":
     main()
